<?php

/**
 * webtrees: online genealogy
 * Copyright (C) 2022 webtrees development team
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 */

declare(strict_types=1);

namespace Fisharebest\Webtrees\CommonMark;

use Fisharebest\Webtrees\Tree;
use League\CommonMark\ConfigurableEnvironmentInterface;
use League\CommonMark\Extension\ExtensionInterface;

/**
 * Convert XREFs within markdown text to links
 */
class XrefExtension implements ExtensionInterface
{
    /** @var Tree - match XREFs in this tree */
    private $tree;

    /**
     * MarkdownXrefParser constructor.
     *
     * @param Tree $tree Match XREFs in this tree
     */
    public function __construct(Tree $tree)
    {
        $this->tree = $tree;
    }

    public function register(ConfigurableEnvironmentInterface $environment): void
    {
        $environment
            ->addInlineParser(new XrefParser($this->tree))
            ->addInlineRenderer(XrefNode::class, new XrefRenderer());
    }
}
